--[[ NavHelper by MrInfernal  v2.3

GUI to improve usage of MQ2Nav and streamline travel
Allows the user to search by zone full name, short name, or ID and returns
a selectable table. Click a zone and you can Start, Pause, or Cancel travel.

Buttons to check for teleport locations near your selected zone and to cast the port spell.
]]

---@type Mq
local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

-- Import the zone and port files
local zones = require('navzones')
DruPorts = require('druports')
WizPorts = require('wizports')

AllPorts = {}
for _, addwiz in ipairs(WizPorts) do
    table.insert(AllPorts, addwiz)
end
for _, adddru in ipairs(DruPorts) do
    table.insert(AllPorts, adddru)
end

-- Set default values
local zoneSearch = ""       -- when 'Search' is clicked
local results = {}          -- results from zoneLookup()
local portresults = {}      -- when 'Check' is clicked [Searches the zone for nearby ports]
local destZone = ""         -- the zone chosen after Searching
local portChecked = false   -- keep port results table from showing until data is available
local portSpell = ""        -- the port spell to cast
local groupTravel = 1       -- If navigation commands should be send to DanNet group
local travelChoices = {'Self', 'Group', 'in Zone', 'All'}
local useCWTN = false       -- Disable if not using CWTN Druid
local characterChoices = {"Druid", "Wizard", "Both"}    -- Options for the settings combobox
ClassPorts = 1     -- Combobox selection (1: Dru, 2: Wiz, 3: Both")
local navPause = false      -- Toggles the Pause/Resume button for travelling
local myPorters = {}

-- Checks if you're in a group
if mq.TLO.Group() then
    groupTravel = 2
else
    groupTravel = 1
end

-- Add druids and wizards to a DanNet channel called porters
if mq.TLO.Me.Class() == 'Druid' then
    mq.cmd('/squelch /djoin porters')
elseif mq.TLO.Me.Class() == 'Wizard' then
    mq.cmd('/squelch /djoin porters')
else
    mq.cmd('/squelch /dge dru /djoin porters')
    mq.cmd('/squelch /dge wiz /djoin porters')
end

-- Create a table of the porter character name and class checks
local function setMyPorters(porters)
    local myPorters = {}

    if mq.TLO.Group() then
        for char in porters:gmatch("[^|]+") do
            local name = mq.TLO.Group.Member(char).Name()
            local class = mq.TLO.Group.Member(char).Class()
            local level = mq.TLO.Group.Member(char).Level()
            table.insert(myPorters, {name = name, class = class, level = level})
        end
    else
        for char2 in porters:gmatch("[^|]+") do
            local name = mq.TLO.Me.Name()
            local class = mq.TLO.Me.Class()
            local level = mq.TLO.Me.Level()
            table.insert(myPorters, {name = name, class = class, level = level})
        end
    end

    return myPorters
end

-- Use the myPorters table to check for port classes
-- This will set which port spells display (Dru/Wiz/Both)
local myPorters = setMyPorters(tostring(mq.TLO.DanNet.Peers))

local hasDruid = false
local hasWizard = false

for i, porter in ipairs(myPorters) do
    if porter.class == "Druid" then
        ClassPorts = 1
        hasDruid = true
    elseif porter.class == "Wizard" then
        ClassPorts = 2
        hasWizard = true
    end
end

if hasDruid and hasWizard then
    ClassPorts = 3
end

local function checkCWTN()
    if mq.TLO.Plugin('mq2druid').IsLoaded() and hasDruid then
        useCWTN = true
    else
        useCWTN = false
    end
end

-- takes the input field and searches through the navzones for matches
-- can match the full name, short name, or id
local function zoneLookup(input)
    local results = {}

    -- Convert the input to lowercase for case-insensitive comparison
    local lowerInput = string.lower(input)

    -- Checks for zones that match the input
    for _, zone in ipairs(zones) do
        local lowerName = string.lower(zone.name)
        local lowerShortName = string.lower(zone.shortname)

        if string.find(lowerName, lowerInput) or
           string.find(lowerShortName, lowerInput) or
           tostring(zone.id) == input then
            table.insert(results, zone)
        end
    end

    return results
end

--[[ Checks the selected zone for nearby port spell locations
    Zoneports have the direct location of the spell, as well as the surrounding zones
    Function to set ports based on classPorts setting (Dru/Wiz/Both)
    Results are based on if the port matches the zone selected:
        If it is a match (i.e. a direct port), only those spells display
        Otherwise, it will search for a spell that goes to a connected zone
--]] 
local function portCheck(input)
    local ports = {}
    local results = {}

    if ClassPorts == 1 then
        ports = DruPorts
    elseif ClassPorts == 2 then
        ports = WizPorts
    elseif ClassPorts == 3 then
        ports = AllPorts
    end

    for _, port in ipairs(ports) do
        if port.loc == input then
            table.insert(results, port)
        end
    end

    if (classPorts ~= 3 and #results < 2) or (classPorts == 3 and #results < 4) then
        for _, port in ipairs(ports) do
            if string.find(table.concat(port.connections,','), input) then
                table.insert(results, port)
            end
        end
    end

    return results
end

local function doEvac()
    local evacSpells = {'Exodus', 'Succor', 'Lesser Succor', 'Evacuate', 'Lesser Evacuate'}
    local portName = ''
    local myPorters = setMyPorters(tostring(mq.TLO.DanNet.Peers))

    for _, char in ipairs(myPorters) do
        for _, port in ipairs(evacSpells) do
            if char.class == 'Druid' and char.level >= 57 and port == 'Succor' then
                portName = port
                break
            elseif char.class == 'Druid' and char.level >= 18 and port == 'Lesser Succor' then
                portName = port
                break
            elseif char.class == 'Wizard' and char.level >= 57 and port == 'Evacuate' then
                portName = port
                break
            elseif char.class == 'Wizard' and char.level >= 18 and port == 'Lesser Evacuate' then
                portName = port
                break
            end
        end
    end

    if hasDruid and useCWTN then
        mq.cmd('/squelch /dgae porters /dru pause on')
    end

    -- Have the porter cast the spell. Will report if need to memorize or don't have the spell
    mq.cmd('/noparse /squelch /dgae porters /if (${Cast.Ready[exodus]}) /casting "exodus" alt')
    mq.cmdf('/timed 1 /noparse /squelch /dgae porters /if (${Me.Book["%s"]}>0) /casting "%s"', portName, portName)
    mq.cmdf('/timed 2 /noparse /squelch /dgae porters /if (!${Me.Book["%s"]}>0) /dgt all Spell not known!', portName)

    -- Have to memorize
    mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CM]}) /timed 5 /dgt all Memorizing "%s"! Ready in 1.5 seconds', portName)
    mq.cmdf([[/timed 50 /noparse /squelch /dgae porters /if (${Cast.Timing}>6000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!]], portName)
    mq.cmdf([[/timed 90 /noparse /squelch /dgae porters /if (${Cast.Timing}>4000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!]], portName)

    -- Already memorized
    mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Timing}>8000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', portName)
    mq.cmdf('/timed 40 /noparse /squelch /dgae porters /if (${Cast.Timing}>5000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!', portName)

    -- Out of mana
    mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /dgt all Out of mana!')
    --mq.cmdf('/timed 8 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /if (!${Me.Sitting}) /sit')
end

-- Create a function to handle the teleporting
local function boundPort(input)
    local found = false
    local portName = ''

    for _, port in ipairs(AllPorts) do
        if string.lower(port.name) == string.lower(input) then
            found = true
            portName = port.name
            break
        end
    end

    if found then
        if useCWTN then
            if mq.TLO.Me.Class == 'Druid' then
                mq.cmd('/squelch /dgae porters /dru pause on')
                mq.cmd('/dgt all Pausing MQ2Druid!')
            else
                if hasDruid then
                    mq.cmd('/squelch /dge porters /dru pause on')
                    mq.cmd('/dgt all Pausing MQ2Druid!')
                end
            end
        end
        if mq.TLO.Me.Class == 'Druid' then
            mq.cmdf('/squelch /dgae porters /casting "%s"', portName)
            mq.cmdf('/dgt all Casting "%s"!', portName)
        else
            -- Have the porter cast the spell. Will report if need to memorize or don't have the spell
            mq.cmdf('/noparse /squelch /dgae porters /if (${Me.Book["%s"]}>0) /casting "%s"', portName, portName)
            mq.cmdf('/timed 2 /noparse /squelch /dgae porters /if (!${Me.Book["%s"]}>0) /dgt all Spell not known!', portName)

            -- Have to memorize
            mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CM]}) /timed 5 /dgt all Memorizing "%s"! Ready in 10 seconds', portName)
            mq.cmdf([[/timed 140 /noparse /squelch /dgae porters /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!]], portName)

            -- Already memorized
            mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', portName)
            mq.cmdf('/timed 80 /noparse /squelch /dgae porters /if (${Cast.Timing}>5000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!', portName)

            -- Out of mana
            mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /dgt all Out of mana!')
            mq.cmdf('/timed 8 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /if (!${Me.Sitting}) /sit')
        end
    else
        print('Spell not found!')
    end
end

-- Commands to run after zoning
local function justZoned()
    if useCWTN then
        if mq.TLO.Me.Class() == 'Druid' then
            mq.cmd('/squelch /dgae porters /dru pause off')
            mq.cmd('/dgt all Unpausing MQ2Druid!')
        else
            for _, char in ipairs(myPorters) do
                if char.class == 'Druid' then
                    mq.cmdf('/squelch /dex %s /dru pause off', char.name)
                    mq.cmdf('/squelch /dex %s /dgt all Unpausing MQ2Druid!', char.name)
                --[[elseif char.class == 'Wizard' then
                    mq.cmdf('/squelch /dex %s /rgstart', char.name)
                    mq.cmdf('/squelch /dex %s /dgt all Restarting RGMercs macro!', char.name)]]
                end
            end
        end
    end
end

local function justGrouped(line)
    if groupTravel == 1 then
        groupTravel = 2
    end
end
-------------- End of functions

-- main function to run the window
local function navHelper(open)
    -- Set default position/size
    local main_viewport = imgui.GetMainViewport()
    imgui.SetNextWindowPos(main_viewport.WorkPos.x + 650, main_viewport.WorkPos.y + 20, ImGuiCond.FirstUseEver)
    imgui.SetNextWindowSize(1550, 850, ImGuiCond.FirstUseEver)

    local show = false
    open, show = imgui.Begin("NavHelper", open)

    if not show then
        ImGui.End()
        return open
    end

    ImGui.PushItemWidth(ImGui.GetFontSize() * -12);

    -- Beginning of window elements
    imgui.Text("Search by zone name, short name, or ID")
    zoneSearch = ImGui.InputText('', zoneSearch)

    imgui.SameLine()
    if ImGui.Button("Search") then
        results = zoneLookup(zoneSearch)
        portChecked = false -- Clear the port info display
    end
    imgui.SameLine()
    if ImGui.Button("Reset") then
        results = {} -- Reset the results
        zoneSearch = "" -- Clear the search input
        portChecked = false -- Clear the port info display
    end

    imgui.Separator()

    imgui.Columns(2)
    --Buttons to Start, Pause, or Cancel travel
    imgui.Text('Travel: ')
    imgui.SameLine()
    if ImGui.Button("Start") then
        local danCmd = '/dgga'
        if destZone ~= "" then
            if groupTravel > 1 then
                if groupTravel == 3 then
                    danCmd = '/dgza'
                elseif groupTravel == 4 then
                    danCmd = '/dgae'
                end
                mq.cmdf("%s /travelto %s", danCmd, destZone)
                navPause = false
            else
                mq.cmd("/travelto ", destZone)
                navPause = false
            end
        else
            print('No zone selected!')
        end
    end
    imgui.SameLine()
    -- Toggles between Pause and Resume
    if not navPause then
        if ImGui.Button("Pause") then
            if groupTravel > 1 then
                local danCmd = '/dgga'
                if groupTravel == 3 then
                    danCmd = '/dgza'
                elseif groupTravel == 4 then
                    danCmd = '/dgae'
                end
                mq.cmdf("%s /nav pause", danCmd)
                navPause = true
            else
                mq.cmd("/nav pause")
                navPause = true
            end
        end
    else
        if ImGui.Button("Resume") then
            if groupTravel > 1 then
                local danCmd = '/dgga'
                if groupTravel == 3 then
                    danCmd = '/dgza'
                elseif groupTravel == 4 then
                    danCmd = '/dgae'
                end
                mq.cmdf("%s /nav pause", danCmd)
                navPause = false
            else
                mq.cmd("/nav pause")
                navPause = false
            end
        end
    end
    imgui.SameLine()
    if ImGui.Button("Stop") then
        if groupTravel > 1 then
            local danCmd = '/dgga'
                if groupTravel == 3 then
                    danCmd = '/dgza'
                elseif groupTravel == 4 then
                    danCmd = '/dgae'
                end
            mq.cmdf("%s /travelto stop", danCmd)
            navPause = false
        else
            mq.cmd("/travelto stop")
            navPause = false
        end
    end 

    -- Buttons to Check for and Cast ports. 
    -- Also pauses CWTN plugins
    imgui.Text('Ports:  ')
    imgui.SameLine()
    if ImGui.Button("Check") then
        if destZone ~= "" then
            portChecked = true
            portresults = portCheck(destZone)
        else
            print('No zone selected!')
        end
    end
    imgui.SameLine()
    if ImGui.Button("Cast") then
        if portSpell == "" then
            print ('No port location selected!')
        else
            boundPort(portSpell)
        end
    end
    imgui.SameLine()
    if imgui.Button('Cancel') then
        if groupTravel > 1 then
            mq.cmd('/squelch /dgae porters /stopcast')
            mq.cmd('/timed 5 /squelch /dgae porters /dgt all Canceling Spell!')
        else
            mq.cmd('/stopcast')
            mq.cmd('/timed 5 /dgt all Canceling Spell!')
        end
    end
    imgui.SameLine()
    if imgui.Button('EVAC!') then
        doEvac()
    end

    ImGui.NextColumn()
    imgui.Text('Travel commands to: ')
    imgui.SameLine()
    groupTravel = imgui.Combo(" ", groupTravel, travelChoices, #travelChoices)
    imgui.Text("Class ports to display:")
    ImGui.SameLine()
    ClassPorts = imgui.Combo("  ", ClassPorts, characterChoices, #characterChoices)
    imgui.Columns(1)

    ImGui.Separator()
    -- Create table headings
    imgui.Text('Zone Name')
    ImGui.SameLine(240)
    imgui.Text('Short Name')
    ImGui.SameLine(400)
    imgui.Text('Zone ID')
    imgui.Text('\n')

    -- Iterate over results and create a selectable row for each
    for _, result in ipairs(results) do
        local isSelected = destZone == result.shortname
        if ImGui.Selectable(result.name, isSelected, ImGuiSelectableFlags.SpanAllColumns) then
            -- Set the selected zone to pass to other functions
            destZone = result.shortname
        end

        -- Display short name in the second column
        ImGui.SameLine(240)
        ImGui.Text(result.shortname)

        -- Display ID in the third column
        ImGui.SameLine(400)
        ImGui.Text(tostring(result.id))
    end

    --Port results. Will only show when results are available
    if portChecked then
        imgui.Text('\n')
        ImGui.Separator()
        imgui.Text('Nearby port locations')
        imgui.Text('\n')
        -- Create table headings
        imgui.Text('Level')
        ImGui.SameLine(60)
        imgui.Text('Spell Name')
        ImGui.SameLine(260)
        imgui.Text('Location')
        ImGui.SameLine(380)
        imgui.Text('Target')
        ImGui.SameLine(460)
        imgui.Text('Class')
        imgui.Text('\n')

        -- Iterate over results and create a selectable row for each
        for _, result2 in ipairs(portresults) do
            if portresults == {} then
                imgui.Text('No port locations found!')
            else
                local label = tostring(result2.level) .. "           " .. result2.name
                local isSelected2 = portSpell == result2.name
                if ImGui.Selectable(label, isSelected2, ImGuiSelectableFlags.SpanAllColumns) then
                    portSpell = result2.name  -- Set the selected spell
                end
                
                ImGui.SameLine(260)
                imgui.Text(result2.loc)
                ImGui.SameLine(380)
                imgui.Text(result2.target)
                ImGui.SameLine(460)
                imgui.Text(result2.class)
            end
        end
    end

    imgui.Spacing()
    imgui.PopItemWidth()
    imgui.End()
    return open
end

-- Creates a slash command for porting. Allows creation of hotkeys
-- ex: /nh "circle of knowledge"
mq.bind("/nh", boundPort)
mq.bind('/nhevac', doEvac)

-- Set up event listeners
mq.event('zoned', 'You have entered#*#', justZoned)
mq.event('grouped', 'You have joined the group#*#', justGrouped)

-- See if MQ2Druid is loaded and a grouped druid. If so, then toggle the useCWTN box in settings
checkCWTN()

local openGUI = true

ImGui.Register('NavHelper', function()
    openGUI = navHelper(openGUI)
end)

while openGUI do
    mq.doevents()
    mq.delay(1000) -- equivalent to '1s'
end